@@ -38,6 +38,9 @@ typedef struct {
 
         SoupMessageMetrics *metrics;
 
+        /* Request body logger */
+        SoupLogger *logger;
+
 #ifdef HAVE_SYSPROF
         gint64 begin_time_nsec;
 #endif
@@ -146,8 +149,11 @@ request_body_stream_wrote_data_cb (SoupMessage *msg,
                         client_io->msg_io->metrics->request_body_size += count;
         }
 
-        if (!is_metadata)
+        if (!is_metadata) {
+                if (client_io->msg_io->logger)
+                        soup_logger_log_request_data (client_io->msg_io->logger, msg, (const char *)buffer, count);
                 soup_message_wrote_body_data (msg, count);
+        }
 }
 
 static void
@@ -252,7 +258,10 @@ write_headers (SoupMessage  *msg,
                 uri_string = g_strdup_printf ("%s:%d", uri_host, g_uri_get_port (uri));
                 g_free (uri_host);
         } else {
-                gboolean proxy = soup_connection_is_via_proxy (soup_message_get_connection (msg));
+                SoupConnection *conn = soup_message_get_connection (msg);
+                gboolean proxy = soup_connection_is_via_proxy (conn);
+
+                g_object_unref (conn);
 
                 /* Proxy expects full URI to destination. Otherwise
                  * just the path.
@@ -354,10 +363,7 @@ io_write (SoupClientMessageIOHTTP1 *client_io,
                 io->write_state = SOUP_MESSAGE_IO_STATE_BODY;
                 logger = soup_session_get_feature_for_message (client_io->msg_io->item->session,
                                                                SOUP_TYPE_LOGGER, msg);
-                if (logger) {
-                        soup_logger_request_body_setup (SOUP_LOGGER (logger), msg,
-                                                        SOUP_BODY_OUTPUT_STREAM (io->body_ostream));
-                }
+                client_io->msg_io->logger = logger ? SOUP_LOGGER (logger) : NULL;
                 break;
 
         case SOUP_MESSAGE_IO_STATE_BODY:
@@ -854,9 +860,12 @@ soup_client_message_io_http1_run (SoupClientMessageIO *iface,
                                        soup_client_message_io_http1_get_priority (client_io));
                 g_source_attach (io->io_source, g_main_context_get_thread_default ());
         } else {
-                if ((SoupClientMessageIOHTTP1 *)soup_message_get_io_data (msg) == client_io)
-                        soup_message_io_finish (msg, error);
-                g_error_free (error);
+                if ((SoupClientMessageIOHTTP1 *)soup_message_get_io_data (msg) == client_io) {
+                        g_assert (!client_io->msg_io->item->error);
+                        client_io->msg_io->item->error = g_steal_pointer (&error);
+                        soup_message_io_finish (msg, client_io->msg_io->item->error);
+                }
+                g_clear_error (&error);
 
         }
 
