@@ -15,44 +15,33 @@
 #include "soup-connection.h"
 #include "soup-server-message-private.h"
 #include "soup-message-headers-private.h"
-#include "soup-socket.h"
 #include "soup-uri-utils-private.h"
 
-/**
- * SECTION:soup-server-message
- * @short_description: An HTTP server request and response.
- * @see_also: #SoupMessageHeaders, #SoupMessageBody
- *
- * A SoupServerMessage represents an HTTP message that is being sent or
- * received on a #SoupServer
- *
- * #SoupServer will create #SoupServerMessage<!-- -->s automatically for
- * incoming requests, which your application will receive via handlers.
- *
- * Note that libsoup's terminology here does not quite match the HTTP
- * specification: in RFC 2616, an "HTTP-message" is
- * <emphasis>either</emphasis> a Request, <emphasis>or</emphasis> a
- * Response. In libsoup, a #SoupServerMessage combines both the request and
- * the response.
- **/
-
 /**
  * SoupServerMessage:
  *
- * Class represnting an HTTP request and response pair for a server.
- */
+ * An HTTP server request and response pair.
+ *
+ * A SoupServerMessage represents an HTTP message that is being sent or
+ * received on a [class@Server].
+ *
+ * [class@Server] will create `SoupServerMessage`s automatically for
+ * incoming requests, which your application will receive via handlers.
+ *
+ * Note that libsoup's terminology here does not quite match the HTTP
+ * specification: in RFC 2616, an "HTTP-message" is *either* a Request, *or* a
+ * Response. In libsoup, a #SoupServerMessage combines both the request and the
+ * response.
+ **/
 
 struct _SoupServerMessage {
         GObject             parent;
 
-        SoupSocket         *sock;
-        GSocket            *gsock;
+        SoupServerConnection *conn;
         SoupAuthDomain     *auth_domain;
         char               *auth_user;
 
-        GSocketAddress     *remote_addr;
         char               *remote_ip;
-        GSocketAddress     *local_addr;
 
         const char         *method;
         SoupHTTPVersion     http_version;
@@ -69,9 +58,12 @@ struct _SoupServerMessage {
         SoupMessageBody    *response_body;
         SoupMessageHeaders *response_headers;
 
-        SoupServerMessageIOData *io_data;
+        SoupServerMessageIO *io_data;
 
         gboolean                 options_ping;
+
+        GTlsCertificate      *tls_peer_certificate;
+        GTlsCertificateFlags  tls_peer_certificate_errors;
 };
 
 struct _SoupServerMessageClass {
@@ -91,6 +83,7 @@ enum {
         GOT_CHUNK,
         GOT_BODY,
 
+        CONNECTED,
         DISCONNECTED,
         FINISHED,
 
@@ -101,6 +94,17 @@ enum {
 
 static guint signals[LAST_SIGNAL] = { 0 };
 
+enum {
+	PROP_0,
+
+	PROP_TLS_PEER_CERTIFICATE,
+	PROP_TLS_PEER_CERTIFICATE_ERRORS,
+
+	LAST_PROPERTY
+};
+
+static GParamSpec *properties[LAST_PROPERTY] = { NULL, };
+
 static void
 soup_server_message_init (SoupServerMessage *msg)
 {
@@ -116,18 +120,13 @@ soup_server_message_finalize (GObject *object)
 {
         SoupServerMessage *msg = SOUP_SERVER_MESSAGE (object);
 
-        soup_server_message_io_data_free (msg->io_data);
-
         g_clear_object (&msg->auth_domain);
         g_clear_pointer (&msg->auth_user, g_free);
-        g_clear_object (&msg->remote_addr);
-        g_clear_object (&msg->local_addr);
 
-        if (msg->sock) {
-                g_signal_handlers_disconnect_by_data (msg->sock, msg);
-                g_object_unref (msg->sock);
+        if (msg->conn) {
+                g_signal_handlers_disconnect_by_data (msg->conn, msg);
+                g_object_unref (msg->conn);
         }
-        g_clear_object (&msg->gsock);
         g_clear_pointer (&msg->remote_ip, g_free);
 
         g_clear_pointer (&msg->uri, g_uri_unref);
@@ -141,12 +140,32 @@ soup_server_message_finalize (GObject *object)
         G_OBJECT_CLASS (soup_server_message_parent_class)->finalize (object);
 }
 
+static void
+soup_server_message_get_property (GObject *object, guint prop_id,
+                                  GValue *value, GParamSpec *pspec)
+{
+	SoupServerMessage *msg = SOUP_SERVER_MESSAGE (object);
+
+	switch (prop_id) {
+	case PROP_TLS_PEER_CERTIFICATE:
+		g_value_set_object (value, msg->tls_peer_certificate);
+		break;
+	case PROP_TLS_PEER_CERTIFICATE_ERRORS:
+		g_value_set_flags (value, msg->tls_peer_certificate_errors);
+		break;
+	default:
+		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
+		break;
+	}
+}
+
 static void
 soup_server_message_class_init (SoupServerMessageClass *klass)
 {
         GObjectClass *object_class = G_OBJECT_CLASS (klass);
 
         object_class->finalize = soup_server_message_finalize;
+        object_class->get_property = soup_server_message_get_property;
 
         /**
          * SoupServerMessage::wrote-informational:
@@ -186,11 +205,11 @@ soup_server_message_class_init (SoupServerMessageClass *klass)
          * Emitted immediately after writing a body chunk for a message.
          *
          * Note that this signal is not parallel to
-         * #SoupServerMessage::got-chunk; it is emitted only when a complete
-         * chunk (added with soup_message_body_append() or
-         * soup_message_body_append_bytes()) has been written. To get
+         * [signal@ServerMessage::got-chunk]; it is emitted only when a complete
+         * chunk (added with [method@MessageBody.append] or
+         * [method@MessageBody.append_bytes] has been written. To get
          * more useful continuous progress information, use
-         * #SoupServerMessage::wrote-body-data.
+         * [signal@ServerMessage::wrote-body-data].
          */
         signals[WROTE_CHUNK] =
                 g_signal_new ("wrote-chunk",
@@ -255,10 +274,10 @@ soup_server_message_class_init (SoupServerMessageClass *klass)
          * @msg: the message
          * @chunk: the just-read chunk
          *
-         * Emitted after receiving a chunk of a message body. Note
-         * that "chunk" in this context means any subpiece of the
-         * body, not necessarily the specific HTTP 1.1 chunks sent by
-         * the other side.
+         * Emitted after receiving a chunk of a message body.
+         *
+         * Note that "chunk" in this context means any subpiece of the body, not
+         * necessarily the specific HTTP 1.1 chunks sent by the other side.
          */
         signals[GOT_CHUNK] =
                 g_signal_new ("got-chunk",
@@ -290,7 +309,7 @@ soup_server_message_class_init (SoupServerMessageClass *klass)
          * @msg: the message
          *
          * Emitted when all HTTP processing is finished for a message.
-         * (After #SoupServerMessage::wrote-body).
+         * (After [signal@ServerMessage::wrote-body]).
          */
         signals[FINISHED] =
                 g_signal_new ("finished",
@@ -300,6 +319,20 @@ soup_server_message_class_init (SoupServerMessageClass *klass)
                               NULL, NULL,
                               NULL,
                               G_TYPE_NONE, 0);
+        /**
+         * SoupServerMessage::connected:
+         * @msg: the message
+         *
+         * Emitted when the @msg's socket is connected and the TLS handshake completed.
+         */
+        signals[CONNECTED] =
+                g_signal_new ("connected",
+                              G_OBJECT_CLASS_TYPE (object_class),
+                              G_SIGNAL_RUN_LAST,
+                              0,
+                              NULL, NULL,
+                              NULL,
+                              G_TYPE_NONE, 0);
 
         /**
          * SoupServerMessage::disconnected:
@@ -328,8 +361,8 @@ soup_server_message_class_init (SoupServerMessageClass *klass)
 	 * @tls_errors.
 	 *
 	 * Returns: %TRUE to accept the TLS certificate and stop other
-	 *     handlers from being invoked, or %FALSE to propagate the
-	 *     event further.
+	 *   handlers from being invoked, or %FALSE to propagate the
+	 *   event further.
 	 */
 	signals[ACCEPT_CERTIFICATE] =
 		g_signal_new ("accept-certificate",
@@ -341,18 +374,59 @@ soup_server_message_class_init (SoupServerMessageClass *klass)
 			      G_TYPE_BOOLEAN, 2,
 			      G_TYPE_TLS_CERTIFICATE,
 			      G_TYPE_TLS_CERTIFICATE_FLAGS);
+
+	/**
+	 * SoupServerMessage:tls-peer-certificate:
+	 *
+	 * The peer's #GTlsCertificate associated with the message
+	 *
+	 * Since: 3.2
+	 */
+        properties[PROP_TLS_PEER_CERTIFICATE] =
+		g_param_spec_object ("tls-peer-certificate",
+				     "TLS Peer Certificate",
+				     "The TLS peer certificate associated with the message",
+				     G_TYPE_TLS_CERTIFICATE,
+				     G_PARAM_READABLE |
+				     G_PARAM_STATIC_STRINGS);
+
+	/**
+	 * SoupServerMessage:tls-peer-certificate-errors:
+	 *
+	 * The verification errors on #SoupServerMessage:tls-peer-certificate
+	 *
+	 * Since: 3.2
+	 */
+        properties[PROP_TLS_PEER_CERTIFICATE_ERRORS] =
+		g_param_spec_flags ("tls-peer-certificate-errors",
+				    "TLS Peer Certificate Errors",
+				    "The verification errors on the message's TLS peer certificate",
+				    G_TYPE_TLS_CERTIFICATE_FLAGS, 0,
+				    G_PARAM_READABLE |
+				    G_PARAM_STATIC_STRINGS);
+
+        g_object_class_install_properties (object_class, LAST_PROPERTY, properties);
 }
 
 static void
-socket_disconnected (SoupServerMessage *msg)
+connection_connected (SoupServerMessage *msg)
 {
+        g_assert (!msg->io_data);
+        msg->io_data = soup_server_connection_get_io_data (msg->conn);
+        g_signal_emit (msg, signals[CONNECTED], 0);
+}
+
+static void
+connection_disconnected (SoupServerMessage *msg)
+{
+        msg->io_data = NULL;
         g_signal_emit (msg, signals[DISCONNECTED], 0);
 }
 
 static gboolean
-socket_accept_certificate (SoupServerMessage    *msg,
-                           GTlsCertificate      *tls_certificate,
-                           GTlsCertificateFlags *tls_errors)
+connection_accept_certificate (SoupServerMessage    *msg,
+                               GTlsCertificate      *tls_certificate,
+                               GTlsCertificateFlags *tls_errors)
 {
 	gboolean accept = FALSE;
 
@@ -361,22 +435,51 @@ socket_accept_certificate (SoupServerMessage    *msg,
 	return accept;
 }
 
+static void
+soup_server_message_set_tls_peer_certificate (SoupServerMessage   *msg,
+                                              GTlsCertificate     *tls_certificate,
+                                              GTlsCertificateFlags tls_errors)
+{
+        if (msg->tls_peer_certificate == tls_certificate && msg->tls_peer_certificate_errors == tls_errors)
+                return;
+
+        g_clear_object (&msg->tls_peer_certificate);
+        msg->tls_peer_certificate = tls_certificate ? g_object_ref (tls_certificate) : NULL;
+        msg->tls_peer_certificate_errors = tls_errors;
+        g_object_notify_by_pspec (G_OBJECT (msg), properties[PROP_TLS_PEER_CERTIFICATE]);
+        g_object_notify_by_pspec (G_OBJECT (msg), properties[PROP_TLS_PEER_CERTIFICATE_ERRORS]);
+}
+
+static void
+re_emit_tls_certificate_changed (SoupServerMessage    *msg,
+                                 GParamSpec           *pspec,
+                                 SoupServerConnection *conn)
+{
+        soup_server_message_set_tls_peer_certificate (msg,
+                                                      soup_server_connection_get_tls_peer_certificate (conn),
+                                                      soup_server_connection_get_tls_peer_certificate_errors (conn));
+}
+
 SoupServerMessage *
-soup_server_message_new (SoupSocket *sock)
+soup_server_message_new (SoupServerConnection *conn)
 {
         SoupServerMessage *msg;
 
         msg = g_object_new (SOUP_TYPE_SERVER_MESSAGE, NULL);
-        msg->sock = g_object_ref (sock);
-        msg->gsock = soup_socket_get_gsocket (sock);
-        if (msg->gsock)
-                g_object_ref (msg->gsock);
+        msg->conn = g_object_ref (conn);
+        msg->io_data = soup_server_connection_get_io_data (msg->conn);
 
-        g_signal_connect_object (sock, "disconnected",
-                                 G_CALLBACK (socket_disconnected),
+        g_signal_connect_object (conn, "connected",
+                                 G_CALLBACK (connection_connected),
                                  msg, G_CONNECT_SWAPPED);
-        g_signal_connect_object (sock, "accept-certificate",
-                                 G_CALLBACK (socket_accept_certificate),
+        g_signal_connect_object (conn, "disconnected",
+                                 G_CALLBACK (connection_disconnected),
+                                 msg, G_CONNECT_SWAPPED);
+        g_signal_connect_object (conn, "accept-certificate",
+                                 G_CALLBACK (connection_accept_certificate),
+                                 msg, G_CONNECT_SWAPPED);
+        g_signal_connect_object (conn, "notify::tls-certificate",
+                                 G_CALLBACK (re_emit_tls_certificate_changed),
                                  msg, G_CONNECT_SWAPPED);
 
         return msg;
@@ -391,10 +494,10 @@ soup_server_message_set_uri (SoupServerMessage *msg,
         msg->uri = soup_uri_copy_with_normalized_flags (uri);
 }
 
-SoupSocket *
-soup_server_message_get_soup_socket (SoupServerMessage *msg)
+SoupServerConnection *
+soup_server_message_get_connection (SoupServerMessage *msg)
 {
-        return msg->sock;
+        return msg->conn;
 }
 
 void
@@ -414,6 +517,9 @@ soup_server_message_set_auth (SoupServerMessage *msg,
 gboolean
 soup_server_message_is_keepalive (SoupServerMessage *msg)
 {
+        if (msg->http_version == SOUP_HTTP_2_0)
+                return TRUE;
+
         if (msg->status_code == SOUP_STATUS_OK && msg->method == SOUP_METHOD_CONNECT)
                 return TRUE;
 
@@ -449,19 +555,76 @@ soup_server_message_is_keepalive (SoupServerMessage *msg)
 }
 
 void
-soup_server_message_set_io_data (SoupServerMessage       *msg,
-                                 SoupServerMessageIOData *io)
+soup_server_message_read_request (SoupServerMessage        *msg,
+                                  SoupMessageIOCompletionFn completion_cb,
+                                  gpointer                  user_data)
 {
-        soup_server_message_io_data_free (msg->io_data);
-        msg->io_data = io;
+        soup_server_message_io_read_request (msg->io_data, msg, completion_cb, user_data);
 }
 
-SoupServerMessageIOData *
+SoupServerMessageIO *
 soup_server_message_get_io_data (SoupServerMessage *msg)
 {
         return msg->io_data;
 }
 
+/**
+ * soup_server_message_pause:
+ * @msg: a SoupServerMessage
+ *
+ * Pauses I/O on @msg.
+ *
+ * This can be used when you need to return from the server handler without
+ * having the full response ready yet. Use [method@ServerMessage.unpause] to
+ * resume I/O.
+ *
+ * Since: 3.2
+ */
+void
+soup_server_message_pause (SoupServerMessage *msg)
+{
+        g_return_if_fail (SOUP_IS_SERVER_MESSAGE (msg));
+        g_return_if_fail (msg->io_data != NULL);
+
+        soup_server_message_io_pause (msg->io_data, msg);
+}
+
+/**
+ * soup_server_message_unpause:
+ * @msg: a SoupServerMessage
+ *
+ * Resumes I/O on @msg.
+ *
+ * Use this to resume after calling [method@ServerMessage.pause], or after
+ * adding a new chunk to a chunked response. I/O won't actually resume until you
+ * return to the main loop.
+ *
+ * Since: 3.2
+ */
+void
+soup_server_message_unpause (SoupServerMessage *msg)
+{
+        g_return_if_fail (SOUP_IS_SERVER_MESSAGE (msg));
+        g_return_if_fail (msg->io_data != NULL);
+
+        soup_server_message_io_unpause (msg->io_data, msg);
+}
+
+gboolean
+soup_server_message_is_io_paused (SoupServerMessage *msg)
+{
+        return msg->io_data && soup_server_message_io_is_paused (msg->io_data, msg);
+}
+
+void
+soup_server_message_finish (SoupServerMessage *msg)
+{
+        if (!msg->io_data)
+                return;
+
+        soup_server_message_io_finished (g_steal_pointer (&msg->io_data), msg);
+}
+
 void
 soup_server_message_cleanup_response (SoupServerMessage *msg)
 {
@@ -680,7 +843,7 @@ soup_server_message_set_http_version (SoupServerMessage *msg,
  * soup_server_message_get_reason_phrase:
  * @msg: a #SoupServerMessage:
  *
- * Get the HTTP reason phrase of @msg or %NULL.
+ * Get the HTTP reason phrase of @msg.
  *
  * Returns: (nullable): the reason phrase.
  */
@@ -714,9 +877,10 @@ soup_server_message_get_status (SoupServerMessage *msg)
  * @status_code: an HTTP status code
  * @reason_phrase: (nullable): a reason phrase
  *
- * Sets @msg's status code to @status_code. If @status_code is a
- * known value and @reason_phrase is %NULL, the reason_phrase will
- * be set automatically.
+ * Sets @msg's status code to @status_code.
+ *
+ * If @status_code is a known value and @reason_phrase is %NULL, the
+ * reason_phrase will be set automatically.
  **/
 void
 soup_server_message_set_status (SoupServerMessage *msg,
@@ -792,13 +956,13 @@ soup_server_message_set_response (SoupServerMessage *msg,
  * @redirect_uri: the URI to redirect @msg to
  *
  * Sets @msg's status_code to @status_code and adds a Location header
- * pointing to @redirect_uri. Use this from a #SoupServer when you
+ * pointing to @redirect_uri. Use this from a [class@Server] when you
  * want to redirect the client to another URI.
  *
  * @redirect_uri can be a relative URI, in which case it is
  * interpreted relative to @msg's current URI. In particular, if
  * @redirect_uri is just a path, it will replace the path
- * <emphasis>and query</emphasis> of @msg's URI.
+ * *and query* of @msg's URI.
  */
 void
 soup_server_message_set_redirect (SoupServerMessage *msg,
@@ -825,7 +989,7 @@ soup_server_message_set_redirect (SoupServerMessage *msg,
  * soup_server_message_get_socket:
  * @msg: a #SoupServerMessage
  *
- * Retrieves the #GSocket that @msg is associated with.
+ * Retrieves the [class@Gio.Socket] that @msg is associated with.
  *
  * If you are using this method to observe when multiple requests are
  * made on the same persistent HTTP connection (eg, as the ntlm-test
@@ -835,66 +999,52 @@ soup_server_message_set_redirect (SoupServerMessage *msg,
  * previously-destroyed socket to represent a new socket.
  *
  * Returns: (nullable) (transfer none): the #GSocket that @msg is
- * associated with, %NULL if you used soup_server_accept_iostream().
+ *   associated with, %NULL if you used [method@Server.accept_iostream].
  */
 GSocket *
 soup_server_message_get_socket (SoupServerMessage *msg)
 {
         g_return_val_if_fail (SOUP_IS_SERVER_MESSAGE (msg), NULL);
 
-        return msg->gsock;
+        return soup_server_connection_get_socket (msg->conn);
 }
 
 /**
  * soup_server_message_get_remote_address:
  * @msg: a #SoupServerMessage
  *
- * Retrieves the #GSocketAddress associated with the remote end
+ * Retrieves the [class@Gio.SocketAddress] associated with the remote end
  * of a connection.
  *
  * Returns: (nullable) (transfer none): the #GSocketAddress
- *     associated with the remote end of a connection, it may be
- *     %NULL if you used soup_server_accept_iostream().
+ *   associated with the remote end of a connection, it may be
+ *   %NULL if you used [class@Server.accept_iostream].
  */
 GSocketAddress *
 soup_server_message_get_remote_address (SoupServerMessage *msg)
 {
         g_return_val_if_fail (SOUP_IS_SERVER_MESSAGE (msg), NULL);
 
-        if (msg->remote_addr)
-                return msg->remote_addr;
-
-        msg->remote_addr = msg->gsock ?
-                g_socket_get_remote_address (msg->gsock, NULL) :
-                G_SOCKET_ADDRESS (g_object_ref (soup_socket_get_remote_address (msg->sock)));
-
-        return msg->remote_addr;
+        return soup_server_connection_get_remote_address (msg->conn);
 }
 
 /**
  * soup_server_message_get_local_address:
  * @msg: a #SoupServerMessage
  *
- * Retrieves the #GSocketAddress associated with the local end
+ * Retrieves the [class@Gio.SocketAddress] associated with the local end
  * of a connection.
  *
  * Returns: (nullable) (transfer none): the #GSocketAddress
- *     associated with the local end of a connection, it may be
- *     %NULL if you used soup_server_accept_iostream().
+ *   associated with the local end of a connection, it may be
+ *   %NULL if you used [method@Server.accept_iostream].
  */
 GSocketAddress *
 soup_server_message_get_local_address (SoupServerMessage *msg)
 {
         g_return_val_if_fail (SOUP_IS_SERVER_MESSAGE (msg), NULL);
 
-        if (msg->local_addr)
-                return msg->local_addr;
-
-        msg->local_addr = msg->gsock ?
-                g_socket_get_local_address (msg->gsock, NULL) :
-                G_SOCKET_ADDRESS (g_object_ref (soup_socket_get_local_address (msg->sock)));
-
-        return msg->local_addr;
+        return soup_server_connection_get_local_address (msg->conn);
 }
 
 /**
@@ -905,29 +1055,23 @@ soup_server_message_get_local_address (SoupServerMessage *msg)
  * connection.
  *
  * Returns: (nullable): the IP address associated with the remote
- *     end of a connection, it may be %NULL if you used
- *     soup_server_accept_iostream().
+ *   end of a connection, it may be %NULL if you used
+ *   [method@Server.accept_iostream].
  */
 const char *
 soup_server_message_get_remote_host (SoupServerMessage *msg)
 {
         g_return_val_if_fail (SOUP_IS_SERVER_MESSAGE (msg), NULL);
 
-        if (msg->remote_ip)
-                return msg->remote_ip;
-
-        if (msg->gsock) {
-                GSocketAddress *addr = soup_server_message_get_remote_address (msg);
+        if (!msg->remote_ip) {
+                GSocketAddress *addr = soup_server_connection_get_remote_address (msg->conn);
                 GInetAddress *iaddr;
 
                 if (!addr || !G_IS_INET_SOCKET_ADDRESS (addr))
                         return NULL;
+
                 iaddr = g_inet_socket_address_get_address (G_INET_SOCKET_ADDRESS (addr));
                 msg->remote_ip = g_inet_address_to_string (iaddr);
-        } else {
-                GInetSocketAddress *addr = G_INET_SOCKET_ADDRESS (soup_socket_get_remote_address (msg->sock));
-                GInetAddress *inet_addr = g_inet_socket_address_get_address (addr);
-                msg->remote_ip = g_inet_address_to_string (inet_addr);
         }
 
         return msg->remote_ip;
@@ -937,12 +1081,13 @@ soup_server_message_get_remote_host (SoupServerMessage *msg)
  * soup_server_message_steal_connection:
  * @msg: a #SoupServerMessage
  *
- * "Steals" the HTTP connection associated with @msg from its
- * #SoupServer. This happens immediately, regardless of the current
- * state of the connection; if the response to @msg has not yet finished
- * being sent, then it will be discarded; you can steal the connection from a
- * #SoupServerMessage::wrote-informational or #SoupServerMessage::wrote-body signal
- * handler if you need to wait for part or all of the response to be sent.
+ * "Steals" the HTTP connection associated with @msg from its #SoupServer. This
+ * happens immediately, regardless of the current state of the connection; if
+ * the response to @msg has not yet finished being sent, then it will be
+ * discarded; you can steal the connection from a
+ * [signal@ServerMessage::wrote-informational] or
+ * [signal@ServerMessage::wrote-body] signal handler if you need to wait for
+ * part or all of the response to be sent.
  *
  * Note that when calling this function from C, @msg will most
  * likely be freed as a side effect.
@@ -958,17 +1103,50 @@ soup_server_message_steal_connection (SoupServerMessage *msg)
         GIOStream *stream;
 
         g_object_ref (msg);
-        stream = soup_server_message_io_steal (msg);
-        if (stream) {
-                g_object_set_data_full (G_OBJECT (stream), "GSocket",
-                                        soup_socket_steal_gsocket (msg->sock),
-                                        g_object_unref);
-        }
-
-        g_signal_handlers_disconnect_by_data (msg, msg->sock);
-
-        socket_disconnected (msg);
+        stream = soup_server_connection_steal (msg->conn);
+        g_signal_handlers_disconnect_by_data (msg, msg->conn);
         g_object_unref (msg);
 
         return stream;
 }
+
+/**
+ * soup_server_message_get_tls_peer_certificate:
+ * @msg: a #SoupMessage
+ *
+ * Gets the peer's #GTlsCertificate associated with @msg's connection.
+ * Note that this is not set yet during the emission of
+ * SoupServerMessage::accept-certificate signal.
+ *
+ * Returns: (transfer none) (nullable): @msg's TLS peer certificate,
+ *    or %NULL if @msg's connection is not SSL.
+ *
+ * Since: 3.2
+ */
+GTlsCertificate *
+soup_server_message_get_tls_peer_certificate (SoupServerMessage *msg)
+{
+        g_return_val_if_fail (SOUP_IS_SERVER_MESSAGE (msg), NULL);
+
+        return msg->tls_peer_certificate;
+}
+
+/**
+ * soup_server_message_get_tls_peer_certificate_errors:
+ * @msg: a #SoupMessage
+ *
+ * Gets the errors associated with validating @msg's TLS peer certificate.
+ * Note that this is not set yet during the emission of
+ * SoupServerMessage::accept-certificate signal.
+ *
+ * Returns: a #GTlsCertificateFlags with @msg's TLS peer certificate errors.
+ *
+ * Since: 3.2
+ */
+GTlsCertificateFlags
+soup_server_message_get_tls_peer_certificate_errors (SoupServerMessage *msg)
+{
+        g_return_val_if_fail (SOUP_IS_SERVER_MESSAGE (msg), 0);
+
+        return msg->tls_peer_certificate_errors;
+}
