@@ -69,7 +69,7 @@ soup_add_completion_reffed (GMainContext   *async_context,
 
 /*
  * soup_add_completion: (skip)
- * @async_context: (allow-none): the #GMainContext to dispatch the I/O
+ * @async_context: (nullable): the #GMainContext to dispatch the I/O
  * watch in, or %NULL for the default context
  * @function: the callback to invoke
  * @data: user data to pass to @function
@@ -91,17 +91,17 @@ soup_add_completion (GMainContext *async_context,
 
 /**
  * soup_add_timeout: (skip)
- * @async_context: (allow-none): the #GMainContext to dispatch the I/O
- * watch in, or %NULL for the default context
+ * @async_context: (nullable): the #GMainContext to dispatch the I/O
+ *   watch in, or %NULL for the default context
  * @interval: the timeout interval, in milliseconds
  * @function: the callback to invoke at timeout time
  * @data: user data to pass to @function
  *
- * Adds a timeout as with g_timeout_add(), but using the given
+ * Adds a timeout as with [func@GLib.timeout_add], but using the given
  * @async_context.
  *
  * Returns: (transfer full): a #GSource, which can be removed from @async_context
- * with g_source_destroy().
+ *   with [method@GLib.Source.destroy].
  **/
 GSource *
 soup_add_timeout (GMainContext *async_context,
@@ -114,6 +114,18 @@ soup_add_timeout (GMainContext *async_context,
 	return source;
 }
 
+GMainContext *
+soup_thread_default_context (void)
+{
+        GMainContext *context;
+
+        context = g_main_context_get_thread_default ();
+        if (!context)
+                context = g_main_context_default ();
+
+        return context;
+}
+
 /* 00 URI_UNRESERVED
  * 01 URI_PCT_ENCODED
  * 02 URI_GEN_DELIMS
