@@ -32,15 +32,6 @@
 #include "soup.h"
 #include "soup-misc.h"
 
-/**
- * SECTION:soup-uri-utils
- * @section_id: SoupURIUtils
- * @title: URI Utilities
- * @short_description: Functions to help working with #GUri and HTTP
- *
- * Utility functions and defines to help working with URIs.
- */
-
 /**
  * SOUP_HTTP_URI_FLAGS:
  *
@@ -99,7 +90,7 @@ flags_equal (GUriFlags flags1, GUriFlags flags2)
  * @uri1: a #GUri
  * @uri2: another #GUri
  *
- * Tests whether or not @uri1 and @uri2 are equal in all parts
+ * Tests whether or not @uri1 and @uri2 are equal in all parts.
  *
  * Returns: %TRUE if equal otherwise %FALSE
  **/
@@ -150,9 +141,10 @@ soup_uri_get_path_and_query (GUri *uri)
  * soup_uri_uses_default_port:
  * @uri: a #GUri
  *
- * Tests if @uri uses the default port for its scheme. (Eg, 80 for
- * http.) (This only works for http, https and ftp; libsoup does not know
- * the default ports of other protocols.)
+ * Tests if @uri uses the default port for its scheme.
+ *
+ * (Eg, 80 for http.) (This only works for http, https and ftp; libsoup does not
+ * know the default ports of other protocols.)
  *
  * Returns: %TRUE or %FALSE
  **/
@@ -278,7 +270,7 @@ soup_uri_is_http (GUri *uri)
 /**
  * soup_uri_decode_data_uri:
  * @uri: a data URI, in string form
- * @content_type: (out) (nullable) (transfer full): location to store content type, or %NULL
+ * @content_type: (out) (nullable) (transfer full): location to store content type
  *
  * Decodes the given data URI and returns its contents and @content_type.
  *
@@ -294,6 +286,7 @@ soup_uri_decode_data_uri (const char *uri,
         gboolean base64 = FALSE;
         char *uri_string;
         GBytes *bytes;
+        const char *path;
 
         g_return_val_if_fail (uri != NULL, NULL);
 
@@ -309,8 +302,17 @@ soup_uri_decode_data_uri (const char *uri,
         if (content_type)
                 *content_type = NULL;
 
+        /* g_uri_to_string() is picky about paths that start with `//` and will assert. */
+        path = g_uri_get_path (soup_uri);
+        if (path[0] == '/' && path[1] == '/') {
+                g_uri_unref (soup_uri);
+                return NULL;
+        }
+
         uri_string = g_uri_to_string (soup_uri);
         g_uri_unref (soup_uri);
+        if (!uri_string)
+                return NULL;
 
         start = uri_string + 5;
         comma = strchr (start, ',');
@@ -367,7 +369,7 @@ soup_uri_decode_data_uri (const char *uri,
  * @SOUP_URI_QUERY: the URI query component
  * @SOUP_URI_FRAGMENT: the URI fragment component
  *
- * Enum values passed to soup_uri_copy() to indicate the components of
+ * Enum values passed to [func@uri_copy] to indicate the components of
  * the URI that should be updated with the given values.
  */
 
@@ -378,7 +380,7 @@ soup_uri_decode_data_uri (const char *uri,
  * @...: value of @first_component  followed by additional
  *    components and values, terminated by %SOUP_URI_NONE
  *
- * Return a copy of @uri with the given components updated
+ * Return a copy of @uri with the given components updated.
  *
  * Returns: (transfer full): a new #GUri
  */
