@@ -719,7 +719,7 @@ close_connection (SoupWebsocketConnection *self,
 		code = 0;
 		break;
 	default:
-		if (code < 3000) {
+		if (code < 3000 || code >= 5000) {
 			g_debug ("Wrong closing code %d received", code);
 			protocol_error_and_close (self);
 			return;
@@ -766,6 +766,18 @@ receive_close (SoupWebsocketConnection *self,
 		break;
 	}
 
+        /* 1005, 1006 and 1015 are reserved values and MUST NOT be set as a status code in a Close control frame by an endpoint */
+        switch (priv->peer_close_code) {
+        case SOUP_WEBSOCKET_CLOSE_NO_STATUS:
+        case SOUP_WEBSOCKET_CLOSE_ABNORMAL:
+        case SOUP_WEBSOCKET_CLOSE_TLS_HANDSHAKE:
+                g_debug ("received a broken close frame containing reserved status code %u", priv->peer_close_code);
+                protocol_error_and_close (self);
+                return;
+        default:
+                break;
+        }
+
 	if (len > 2) {
 		data += 2;
 		len -= 2;
@@ -1138,9 +1150,9 @@ soup_websocket_connection_read (SoupWebsocketConnection *self)
 		}
 
 		priv->incoming->len = len + count;
-	} while (count > 0);
 
-	process_incoming (self);
+		process_incoming (self);
+	} while (count > 0 && !priv->close_sent && !priv->io_closing);
 
 	if (end) {
 		if (!priv->close_sent || !priv->close_received) {
