@@ -6,13 +6,14 @@
 #include "test-utils.h"
 
 #include "soup-connection.h"
-#include "soup-socket.h"
+#include "soup-server-connection.h"
 #include "soup-server-message-private.h"
 
 #include <gio/gnetworking.h>
 
 static SoupServer *server;
 static GUri *base_uri;
+static GUri *base_https_uri;
 static GMutex server_mutex;
 
 static void
@@ -24,17 +25,17 @@ forget_close (SoupServerMessage *msg,
 }
 
 static void
-close_socket (SoupServerMessage *msg,
-	      SoupSocket        *sock)
+close_socket (SoupServerMessage    *msg,
+	      SoupServerConnection *conn)
 {
-        GSocket *gsocket;
+        GSocket *socket;
 	int sockfd;
 
 	/* Actually calling soup_socket_disconnect() here would cause
 	 * us to leak memory, so just shutdown the socket instead.
 	 */
-        gsocket = soup_socket_get_gsocket (sock); 
-	sockfd = g_socket_get_fd (gsocket);
+        socket = soup_server_connection_get_socket (conn);
+	sockfd = g_socket_get_fd (socket);
 #ifdef G_OS_WIN32
 	shutdown (sockfd, SD_SEND);
 #else
@@ -50,41 +51,39 @@ close_socket (SoupServerMessage *msg,
 }
 
 static gboolean
-timeout_socket (GObject    *pollable,
-		SoupSocket *sock)
+timeout_socket (GObject              *pollable,
+		SoupServerConnection *conn)
 {
-	soup_socket_disconnect (sock);
+	soup_server_connection_disconnect (conn);
 	return FALSE;
 }
 
 static void
-timeout_request_started (SoupServer        *server,
-			 SoupServerMessage *msg,
-			 gpointer           user_data)
+timeout_request_finished (SoupServer        *server,
+                          SoupServerMessage *msg,
+                          gpointer           user_data)
 {
-	SoupSocket *sock;
-	GMainContext *context = g_main_context_get_thread_default ();
+	SoupServerConnection *conn;
 	GIOStream *iostream;
 	GInputStream *istream;
 	GSource *source;
 
-	g_signal_handlers_disconnect_by_func (server, timeout_request_started, NULL);
+	g_signal_handlers_disconnect_by_func (server, timeout_request_finished, NULL);
 
-	sock = soup_server_message_get_soup_socket (msg);
-	iostream = soup_socket_get_iostream (sock);
+	conn = soup_server_message_get_connection (msg);
+	iostream = soup_server_connection_get_iostream (conn);
 	istream = g_io_stream_get_input_stream (iostream);
 	source = g_pollable_input_stream_create_source (G_POLLABLE_INPUT_STREAM (istream), NULL);
-	g_source_set_callback (source, (GSourceFunc)timeout_socket, sock, NULL);
+	g_source_set_callback (source, (GSourceFunc)timeout_socket, conn, NULL);
 	g_source_attach (source, g_main_context_get_thread_default ());
 	g_source_unref (source);
 
 	g_mutex_unlock (&server_mutex);
-	while (soup_socket_is_connected (sock))
-		g_main_context_iteration (context, TRUE);
 }
 
 static void
-setup_timeout_persistent (SoupServer *server, SoupSocket *sock)
+setup_timeout_persistent (SoupServer           *server,
+                          SoupServerConnection *conn)
 {
 	/* In order for the test to work correctly, we have to
 	 * close the connection *after* the client side writes
@@ -101,8 +100,8 @@ setup_timeout_persistent (SoupServer *server, SoupSocket *sock)
 	 *   3. Close the socket.
 	 */
 	g_mutex_lock (&server_mutex);
-	g_signal_connect (server, "request-started",
-			  G_CALLBACK (timeout_request_started), NULL);
+	g_signal_connect (server, "request-finished",
+			  G_CALLBACK (timeout_request_finished), NULL);
 }
 
 static void
@@ -143,16 +142,16 @@ server_callback (SoupServer        *server,
 					     "Connection", "close");
 
 		if (too_long) {
-			SoupSocket *sock;
+			SoupServerConnection *conn;
 
 			/* soup-message-io will wait for us to add
 			 * another chunk after the first, to fill out
 			 * the declared Content-Length. Instead, we
 			 * forcibly close the socket at that point.
 			 */
-			sock = soup_server_message_get_soup_socket (msg);
+			conn = soup_server_message_get_connection (msg);
 			g_signal_connect (msg, "wrote-chunk",
-					  G_CALLBACK (close_socket), sock);
+					  G_CALLBACK (close_socket), conn);
 		} else if (no_close) {
 			/* Remove the 'Connection: close' after writing
 			 * the headers, so that when we check it after
@@ -166,10 +165,10 @@ server_callback (SoupServer        *server,
 	}
 
 	if (!strcmp (path, "/timeout-persistent")) {
-		SoupSocket *sock;
+		SoupServerConnection *conn;
 
-		sock = soup_server_message_get_soup_socket (msg);
-		setup_timeout_persistent (server, sock);
+		conn = soup_server_message_get_connection (msg);
+		setup_timeout_persistent (server, conn);
 	}
 
 	soup_server_message_set_status (msg, SOUP_STATUS_OK, NULL);
@@ -260,14 +259,17 @@ request_queued_socket_collector (SoupSession *session,
 }
 
 static void
-do_timeout_test_for_session (SoupSession *session)
+do_timeout_test_for_base_uri (GUri *base_uri)
 {
+        SoupSession *session;
 	SoupMessage *msg;
 	GSocket *sockets[4] = { NULL, NULL, NULL, NULL };
 	GUri *timeout_uri;
 	int i;
 	GBytes *body;
 
+        session = soup_test_session_new (NULL);
+
 	g_signal_connect (session, "request-queued",
 			  G_CALLBACK (request_queued_socket_collector),
 			  &sockets);
@@ -310,23 +312,24 @@ do_timeout_test_for_session (SoupSession *session)
 
 	for (i = 0; sockets[i]; i++)
 		g_object_unref (sockets[i]);
+
+        soup_test_session_abort_unref (session);
 }
 
 static void
 do_persistent_connection_timeout_test (void)
 {
-	SoupSession *session;
-
 	g_test_bug ("631525");
 
-	debug_printf (1, "  Normal session, message API\n");
-	session = soup_test_session_new (NULL);
-	do_timeout_test_for_session (session);
-	soup_test_session_abort_unref (session);
+        debug_printf (1, "  HTTP/1\n");
+        do_timeout_test_for_base_uri (base_uri);
+
+        debug_printf (1, "  HTTP/2\n");
+        do_timeout_test_for_base_uri (base_https_uri);
 }
 
 static void
-do_persistent_connection_timeout_test_with_cancellation (void)
+do_persistent_connection_timeout_test_with_cancellation_for_base_uri (GUri *base_uri)
 {
 	SoupSession *session;
 	SoupMessage *msg;
@@ -409,6 +412,16 @@ do_persistent_connection_timeout_test_with_cancellation (void)
 	soup_test_session_abort_unref (session);
 }
 
+static void
+do_persistent_connection_timeout_test_with_cancellation (void)
+{
+        debug_printf (1, "  HTTP/1\n");
+        do_persistent_connection_timeout_test_with_cancellation_for_base_uri (base_uri);
+
+        debug_printf (1, "  HTTP/2\n");
+        do_persistent_connection_timeout_test_with_cancellation_for_base_uri (base_https_uri);
+}
+
 static GMainLoop *max_conns_loop;
 static int msgs_done;
 static guint quit_loop_timeout;
@@ -1645,6 +1658,45 @@ do_connection_force_http2_test (void)
         soup_test_session_abort_unref (session);
 }
 
+static void
+message_restarted (SoupMessage *msg,
+                   gboolean    *was_restarted)
+{
+        *was_restarted = TRUE;
+}
+
+static void
+do_connection_http_1_1_required_test (void)
+{
+        SoupSession *session;
+        SoupMessage *msg;
+        GBytes *body;
+        gboolean was_restarted = FALSE;
+        GError *error = NULL;
+
+        SOUP_TEST_SKIP_IF_NO_TLS;
+        SOUP_TEST_SKIP_IF_NO_APACHE;
+
+        session = soup_test_session_new (NULL);
+
+        msg = soup_message_new ("GET", "https://127.0.0.1:47525/client-cert");
+        soup_message_set_force_http_version (msg, SOUP_HTTP_2_0);
+        g_signal_connect (msg, "restarted",
+                          G_CALLBACK (message_restarted), &was_restarted);
+        body = soup_test_session_async_send (session, msg, NULL, &error);
+        g_assert_no_error (error);
+        g_assert_cmpuint (soup_message_get_status (msg), ==, 403);
+        g_assert_true (was_restarted);
+        g_assert_nonnull (body);
+        g_bytes_unref (body);
+        g_object_unref (msg);
+
+        while (g_main_context_pending (NULL))
+                g_main_context_iteration (NULL, FALSE);
+
+        soup_test_session_abort_unref (session);
+}
+
 int
 main (int argc, char **argv)
 {
@@ -1653,9 +1705,10 @@ main (int argc, char **argv)
 	test_init (argc, argv, NULL);
 	apache_init ();
 
-	server = soup_test_server_new (SOUP_TEST_SERVER_IN_THREAD);
-	soup_server_add_handler (server, NULL, server_callback, "http", NULL);
+	server = soup_test_server_new (SOUP_TEST_SERVER_IN_THREAD | SOUP_TEST_SERVER_HTTP2);
+	soup_server_add_handler (server, NULL, server_callback, NULL, NULL);
 	base_uri = soup_test_server_get_uri (server, "http", NULL);
+        base_https_uri = soup_test_server_get_uri (server, "https", NULL);
 
 	g_test_add_func ("/connection/content-length-framing", do_content_length_framing_test);
 	g_test_add_func ("/connection/persistent-connection-timeout", do_persistent_connection_timeout_test);
@@ -1669,6 +1722,7 @@ main (int argc, char **argv)
 	g_test_add_func ("/connection/preconnect", do_connection_preconnect_test);
         g_test_add_func ("/connection/metrics", do_connection_metrics_test);
         g_test_add_func ("/connection/force-http2", do_connection_force_http2_test);
+        g_test_add_func ("/connection/http2/http-1-1-required", do_connection_http_1_1_required_test);
 
 	ret = g_test_run ();
 
